searchNodes=[{"doc":"ECSx is an Entity-Component-System (ECS) framework for Elixir. In ECS: Every game object is an Entity, represented by a unique ID. The data which comprises an Entity is split among many Components. Game logic is split into Systems, which update the Components every server tick. Components are grouped together - with similar Components from other Entities - into categories called Aspects. Systems can then be organized in a way where each one only operates on Components of a specific Aspect. Under the hood, ECSx uses Erlang Term Storage (ETS) to store active Components in memory. A single GenServer manages the ETS tables to ensure strict serializability and customize the run order for Systems.","ref":"ECSx.html","title":"ECSx","type":"module"},{"doc":"Provides an API for working with Components of a specific type. For example, if Entities in your application should have a &quot;color&quot; value, you will create an Aspect called Color . This allows you to add a color Component to an Entity with add_component/1 , query the color value for a given Entity with get_component/1 or get_value/2 , query all Entities which have a color value with get_all/0 , remove the color value from an Entity altogether with remove_component/1 , or test whether an entity has a color with has_component?/1 . Usage Each Aspect should have its own module, which defines a schema for the data, and optional ETS table settings. The schema defines the field names which will be used to reference each value. defmodule MyApp.Aspects.Color do use ECSx.Aspect , schema : { :entity_id , :hue , :saturation , :lightness } end Options :schema - a tuple of field names :table_type - ETS table type. Possible options are :set , :ordered_set , :bag , and :duplicate_bag . Defaults to :set . See below for more information. :read_concurrency - when true , enables read concurrency for the ETS table. Defaults to false :table_type option By default, ECSx creates ETS tables of type :set for your Components. This means that each Entity may have at most one Component of each Aspect. For example, if your application has a &quot;Height&quot; Aspect, Entities would never need more than one Component to model that value. However, if you had an Aspect such as &quot;TakingDamageOverTime&quot;, you might want an Entity to store each source of damage as a separate value, using multiple Components of the same Aspect. To do this, set :table_type to :bag . Example defmodule TakingDamageOverTime do use ECSx.Aspect , schema : { :entity_id , :damage_per_second , :damage_type , :source_id } , table_type : :bag end alias TakingDamageOverTime , as : DOT DOT . add_component ( entity_id : hero . id , damage_per_second : 10 , type : :poison , source_id : spider . id ) DOT . add_component ( entity_id : hero . id , damage_per_second : 25 , type : :fire , source_id : dragon . id ) DOT . get_component ( hero . id ) [ %{ entity_id : ... } , %{ ... } ] DOT . get_value ( hero . id , :damage_per_second ) [ 10 , 25 ] get_component/1 and get_value/2 The standard way to fetch the component(s) from an Entity is using MyAspect.get_component(entity_id) . If your table_type is :set or :ordered_set , this function will return a single Component, or nil if the Entity does not have that Aspect. If the table_type is :bag or :duplicate_bag , then it will return a list containing zero or more Components. Each Component is represented by a map, using the keys provided by the schema. If you only want a single value from the Component, use get_value/2 , passing the field name of the desired value as the second argument. Examples Color . get_component ( entity_id ) %{ entity_id : entity_id , hue : 300 , saturation : 50 , lightness : 45 } Color . get_value ( entity_id , :hue ) 300","ref":"ECSx.Aspect.html","title":"ECSx.Aspect","type":"behaviour"},{"doc":"Creates a new component. Example ArmorRating . add_component ( entity_id : 123 , value : 10 )","ref":"ECSx.Aspect.html#c:add_component/1","title":"ECSx.Aspect.add_component/1","type":"callback"},{"doc":"Gets all existing components of this aspect.","ref":"ECSx.Aspect.html#c:get_all/0","title":"ECSx.Aspect.get_all/0","type":"callback"},{"doc":"Gets an existing component from the table, given its entity ID.","ref":"ECSx.Aspect.html#c:get_component/1","title":"ECSx.Aspect.get_component/1","type":"callback"},{"doc":"Gets a single value from a component.","ref":"ECSx.Aspect.html#c:get_value/2","title":"ECSx.Aspect.get_value/2","type":"callback"},{"doc":"Checks if an entity has one or more components with this aspect.","ref":"ECSx.Aspect.html#c:has_component?/1","title":"ECSx.Aspect.has_component?/1","type":"callback"},{"doc":"Initializes the ETS table to store all components of this aspect.","ref":"ECSx.Aspect.html#c:init/0","title":"ECSx.Aspect.init/0","type":"callback"},{"doc":"Removes any existing components of this aspect from an entity.","ref":"ECSx.Aspect.html#c:remove_component/1","title":"ECSx.Aspect.remove_component/1","type":"callback"},{"doc":"","ref":"ECSx.Aspect.html#t:aspect/0","title":"ECSx.Aspect.aspect/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:id/0","title":"ECSx.Aspect.id/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:t/0","title":"ECSx.Aspect.t/0","type":"type"},{"doc":"","ref":"ECSx.Aspect.html#t:value/0","title":"ECSx.Aspect.value/0","type":"type"},{"doc":"The Manager for your ECSx application. In an ECSx application, the Manager is responsible for: starting up ETS tables for each Aspect, where the Components will be stored keeping track of the Systems to run, and their run order configuring the tick rate for the application running the Systems every tick validating and writing updates from the player clients","ref":"ECSx.Manager.html","title":"ECSx.Manager","type":"module"},{"doc":"","ref":"ECSx.Manager.html#final_tick/1","title":"ECSx.Manager.final_tick/1","type":"function"},{"doc":"","ref":"ECSx.Manager.html#setup/1","title":"ECSx.Manager.setup/1","type":"macro"},{"doc":"","ref":"ECSx.Manager.html#start_link/1","title":"ECSx.Manager.start_link/1","type":"function"},{"doc":"A fragment of game logic which reads and updates Components. Every system must implement a run function. By default the system will run every game tick. To use a longer period between runs, you can pass the option :period . For example, to set a system to run every 5 ticks: use ECSx.System , period : 5","ref":"ECSx.System.html","title":"ECSx.System","type":"behaviour"},{"doc":"","ref":"ECSx.System.html#c:run/0","title":"ECSx.System.run/0","type":"callback"},{"doc":"Generates a new Aspect for an ECSx application. $ mix ecsx.gen.aspect Color entity_id hue saturation lightness The first argument is the name of the Aspect module, followed by the data fields which will make up each Component.","ref":"Mix.Tasks.Ecsx.Gen.Aspect.html","title":"Mix.Tasks.Ecsx.Gen.Aspect","type":"task"},{"doc":"Generates a new System for an ECSx application. $ mix ecsx.gen.system Foo The only argument accepted is a module name for the System.","ref":"Mix.Tasks.Ecsx.Gen.System.html","title":"Mix.Tasks.Ecsx.Gen.System","type":"task"},{"doc":"Generates the manager process which runs an ECSx application. $ mix ecsx.setup This setup will generate manager.ex and empty folders for aspects and systems. If you don't want to generate the folders, you can provide option --no-folders","ref":"Mix.Tasks.Ecsx.Setup.html","title":"Mix.Tasks.Ecsx.Setup","type":"task"},{"doc":"To create an ECSx application, there are a few simple steps: install Elixir + erlang/OTP install Phoenix (optional) create an Elixir/Phoenix project fetch ECSx as a dependency for your project run the ECSx setup","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"If you don't yet have Elixir and erlang/OTP installed on your machine, follow the instructions on the official Installation Page .","ref":"installation.html#elixir-and-erlang-otp","title":"Installation - Elixir and erlang/OTP","type":"extras"},{"doc":"If you plan on hosting your application online, you'll probably want to use Phoenix. You can skip this step if you only want to run the app locally. Otherwise, follow the instructions for Phoenix installation (the tutorial project will assume you are using Phoenix).","ref":"installation.html#phoenix","title":"Installation - Phoenix","type":"extras"},{"doc":"If you are using Phoenix, you'll create your new application with the command $ mix phx.new my_app Or for a regular Elixir application (with supervision tree): $ mix new my_app --sup","ref":"installation.html#create-project","title":"Installation - Create project","type":"extras"},{"doc":"To use the ECSx framework in your application, it should be added to the list of dependencies in my_app/mix.exs : defp deps do [ { :ecsx , &quot;~&gt; 0.1&quot; } ] end Then (from the root directory of your application) run: $ mix deps.get","ref":"installation.html#install-ecsx","title":"Installation - Install ECSx","type":"extras"},{"doc":"With ECSx installed, you can run the setup generator: $ mix ecsx.setup which will create the Manager, and two folders to get your project started. You'll also need to add the Manager to your application's supervision tree: def start ( _type , _args ) do children = [ MyApp.Manager ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end","ref":"installation.html#setup-ecsx","title":"Installation - Setup ECSx","type":"extras"},{"doc":"You should now have everything you need to start building! If you're already familiar with the Entity-Component-System pattern, jump right in to the tutorial project - otherwise, start with our guide on ECS design .","ref":"installation.html#summary","title":"Installation - Summary","type":"extras"},{"doc":"Now that we have an Elixir project with Phoenix, we can get started on building a game with Entity-Component-System architecture. We're going to use the classic Snake as inspiration.","ref":"tutorial.html","title":"Tutorial Project","type":"extras"},{"doc":"First, let's start with a single entity. In ECS, an entity is nothing by itself, so we must start by defining the aspects which will make up the entity. What makes a Snake? Has a physical position in the game world Moves forward constantly Can change direction Tail grows longer over time Kills other snakes when they touch its tail For each of these requirements, we will create an aspect: Position Moving Direction Length OnContact For each of these aspects, we'll define a schema which contains the ID of the entity, and any relevant data: Position: {entity_id, x, y} ex: {123, 0, -50} Moving: {entity_id, speed} ex: {123, 1} Direction: {entity_id, direction} ex: {123, :north} Length: {entity_id, length} ex: {123, 10} OnContact: {entity_id, result_of_contact} ex: {123, :death} We can use the ECSx generators to quickly create the files needed for these aspects: $ mix ecsx.gen.aspect Position entity_id x y Following the above pattern, run mix ecsx.gen.aspect for the remaining four aspects. Next we have to think about the Systems which will organize game logic. What makes a Snake game work? Snakes move forwards every game tick Snakes change direction when a player gives input Snakes get longer over time (or based on other game conditions) When a collision is detected, one or both snakes are removed from the game Each one of these will be the responsibility of a different System: ForwardMovement PlayerInput GrowTail Collision We will generate modules for each of these Systems with mix ecsx.gen.system . For example: $ mix ecsx.gen.system ForwardMovement After generating all four of our Systems, it's time to write the game logic for each one - COMING SOON","ref":"tutorial.html#design-plan","title":"Tutorial Project - Design plan","type":"extras"},{"doc":"Entities, Aspects, and Components Everything in your application is an entity, but in ECS you won't work with these entities directly - instead you will work directly with the individual attributes that an entity might have. These attributes are called aspects, and they are given to an entity by creating a component, which holds, at minimum, the entity's unique ID, but also any extra data which is relevant to that aspect. For example: You're running a simulation of cars on a highway Each car gets its own entity_id e.g. 123 If the car with ID 123 is blue, we give it the Color aspect, stored as a {123, &quot;blue&quot;} component If the same car is moving west at 60mph, we might model this with a Moving aspect, stored as a {123, 60, &quot;west&quot;} component Another car with ID 135 might have the Moving aspect with different data {135, 30, &quot;east&quot;} These cars would also have Location which holds x, y, z coordinates: {entity_id, x, y, z} Systems Once your data is modeled using components to associate different aspects to your entities, you'll create systems to operate on them. For example: Entities with the Moving aspect should have their locations regularly updated according to the speed and direction We can create a Move system which reads the Moving component, calculates how far the car has moved since the last server tick, and updates the entity's Location component accordingly. The system will run every tick, only considering entities which have the Moving aspect","ref":"ecs_design.html","title":"ECS Design","type":"extras"}]